////////////////////////////////////////////////////////////////////////////////////////
// Module name: FW_Nixie-MQTT.ino
// Project: NixieKlok-MQTT
// by Jan Breemer  jan@breem.nl
//
// Functions in this module:
// ReadBME280 ()
// I2C_WriteByte ()
// Nibbles ()
// TestAdresses()
// DisplayData ()
// Synchronize()
// UpdateClock ()
// ISR(TIMER2_COMPA_vect)
// setup()
// loop()
////////////////////////////////////////////////////////////////////////////////////////

#include "Globals.h"
#include <Wire.h>
#include "Globals.h"
#include <Adafruit_BME280.h>

#define VERSION "NixieKlok 2022-02-08"

// Target Arduino Ethernet
// pin use:
// pin A5  PORTC5 SCL
// pin A4  PORTC4 SDA

#define NIXADDRH 0x20 // PCF 8574 base address for Hours
#define NIXADDRM 0x21 // PCF 8574 base address for Minutes
#define NIXADDRS 0x22 // PCF 8574 base address for Seconds
#define NIXADDRD 0x23 // PCF 8574 base address for Day of Month
#define NIXADDRT 0x24 // PCF 8574 base address for Temperature
#define NIXADDRY 0x25 // PCF 8574 base address for Hygro

byte Minute, Hour, Day, DayOfWeek, Month, Year, Second; // Count Values
byte dMinute, dHour, dDay, dDayOfWeek, dMonth, dYear, dSecond; // display values
bool DoSerial, DoReadBME280, DoDisplayData;

Adafruit_BME280 bme;
bool BME280_IsInitialized;
int Temperature, Hygro;
long Baro;

//******************************************************************************************************
void ReadBME280 (void) // Called once per second from loop()
{
  Temperature = round(bme.readTemperature() * 10);
  Hygro = round(bme.readHumidity()); // % relative
  if (Hygro > 99) Hygro = 99;
  Baro = bme.readPressure(); //Pascal
  if (false)
  {
    Serial.print   ("Temperature = ");
    Serial.print   (Temperature / 10);
    Serial.print   (" Hygro = ");
    Serial.print   (Hygro);
    Serial.print   (" Pressure = ");
    Serial.println (Baro);
  }
  DoPublishClimate = true;
}

//******************************************************************************************************
void I2C_WriteByte (byte Addr, byte Data)
{
  Wire.beginTransmission (Addr);
  Wire.write (Data); 
  Wire.endTransmission();
}

//******************************************************************************************************
byte Nibbles(byte Number) // converts Number to two BCD nibbles 
{
  if(Number > 99) return(0);
  return(((Number / 10) << 4) + (Number % 10)); 
}

//******************************************************************************************************
void TestAdresses()  // diagnostics only
{
  I2C_WriteByte(NIXADDRH, Nibbles(11));
  I2C_WriteByte(NIXADDRM, Nibbles(22));
  I2C_WriteByte(NIXADDRS, Nibbles(33));
  I2C_WriteByte(NIXADDRD, Nibbles(44));
  I2C_WriteByte(NIXADDRT, Nibbles(55));
  I2C_WriteByte(NIXADDRY, Nibbles(66));
}

//********************************************************************************************************
void DisplayData (void)
{
  I2C_WriteByte(NIXADDRS, Nibbles(dSecond));
  I2C_WriteByte(NIXADDRM, Nibbles(dMinute));
  I2C_WriteByte(NIXADDRH, Nibbles(dHour));
  I2C_WriteByte(NIXADDRD, Nibbles(dDay));
  I2C_WriteByte(NIXADDRT, Nibbles(Temperature/10));
  I2C_WriteByte(NIXADDRY, Nibbles(Hygro));
}

//********************************************************************************************************
void Synchronize(void) // synchronize display data with received data
{
  dSecond = Second;
  dMinute = Minute;
  dHour = Hour;
  dDay = Day;
  dDayOfWeek = DayOfWeek;
  dMonth = Month; 
  dYear = Year;
}

//********************************************************************************************************
void UpdateClock (void) 
{
  if(++dSecond > 59)
  {
    dSecond = 0;
    if(++dMinute > 59)
    {
      dMinute = 0;
      if(++dHour > 23)
      {
        dHour = 0;
        if(dDay > 31)
        {
          dDay = 1;
        }
      }
    }
  }
}

//******************************************************************************************************
ISR(TIMER2_COMPA_vect) // interrupt at 10 msec interval = 100Hz.
{
  static byte Counter1sec;
  DIAGPIN_2_T
  if(++Counter1sec > 99)
  {
    Counter1sec = 0; 
    UpdateClock();
    DoDisplayData = true;
    DoSerial = true;
    DoReadBME280 = true;
    if (DoSynchronize) { Synchronize(); DoSynchronize = false;}
  } // ticks of 1 second
}

//******************************************************************************************************
void setup()
{
  Serial.begin(19200); 
  Serial.println();
  Serial.println(VERSION); 
  
  // configure 8 bit Timer 2 for 100 Hz interrupts
  TCCR2A = 0x02;  // waveformMode 2 = CTC
  TCCR2B = 0x07;  // prescaler div 1024, WaveformMode 4 = CTC
  OCR2A = 155;    // for 100 Hz
  TIMSK2 = 0x02;  // interrupt on OCRA match  

  Wire.begin();
  //TestAdresses();

  InitClient();

  Serial.print("Initialize BME280 sensor for Temperature, Hygro and Baro.....");
  BME280_IsInitialized = true;
  if (!bme.begin())
  {
    Serial.println("Could not find a valid BME280 sensor, check wiring!");
    BME280_IsInitialized = false;
  }
  else Serial.println("BME280 is OK");

  UseDIAGPIN_2
  UseDIAGPIN_3
  UseDIAGPIN_4
  UseDIAGPIN_5
}

//******************************************************************************************************
void loop()
{
  // Certain functions do not work if they are called from within an 
  // interrupt handler, so they are called indirectly from loop()
  if (DoReadBME280)
  {
    DoReadBME280 = false;
    ReadBME280();
  }

  if (DoDisplayData)
  {
    DoDisplayData = false;
    DisplayData();
  }
  
  if(DoSerial)
  {
    DoSerial = false;
    return;
    Serial.print(Year);      Serial.print("-");
    Serial.print(Month);     Serial.print("-");
    Serial.print(Day);       Serial.print("--");
    Serial.print(DayOfWeek); Serial.print("--");
    Serial.print(Hour);      Serial.print(":");
    Serial.print(Minute);    Serial.print(":");
    Serial.print(Second);    Serial.print(" ");
    switch(dDayOfWeek)
    {
      case 0: Serial.print("geen dag "); break;
      case 1: Serial.print("zondag "); break;
      case 2: Serial.print("maandag "); break;
      case 3: Serial.print("dinsdag "); break;
      case 4: Serial.print("woensdag "); break;
      case 5: Serial.print("donderdag "); break;
      case 6: Serial.print("vrijdag "); break;     
      case 7: Serial.print("zaterdag "); break;
     
    }
    Serial.print(dYear);       Serial.print("-");
    Serial.print(dMonth);      Serial.print("-");
    Serial.print(dDay);        Serial.print("--");
    Serial.print(dDayOfWeek);  Serial.print("--");
    Serial.print(dHour);       Serial.print(":");
    Serial.print(dMinute);     Serial.print(":");
    Serial.print(dSecond);   Serial.print(" t=");
    Serial.print(Temperature/10); Serial.print(" h=");
    Serial.print(Hygro); Serial.print(" b=");
    Serial.println(Baro);
  }
  DIAGPIN_4_T
  UpdateClient();
  delay(1);    //ca 1 kHz
}
